import OpenGL
OpenGL.USE_ACCELERATE = True
from OpenGL.GL import *
import texture
import screen_rect
import shader_utils

def clear(color):
    glClearColor(color[0]/255, color[1]/255, color[2]/255, 1)
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)

class Shader:
    def __init__(self, size, display, pos, vertex_path, fragment_path):
        self.render_rect = screen_rect.ScreenRect(size, display, pos)

        glEnable(GL_BLEND)
        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
        
        self.shader_data = {}
        self.shader = shader_utils.create_shader(vertex_path, fragment_path)
        
    def send(self, name, data):
        if name in self.shader_data:
            if [float(x) for x in data] == self.shader_data[name]:
                return
        self.shader_data[name] = [float(x) for x in data]

    def render(self, surface=None):
        if surface is not None:
            screen_texture = texture.Texture(surface)
            screen_texture.use()

        glUseProgram(self.shader)
        for key in self.shader_data.keys():
            data = self.shader_data[key]
            loc = glGetUniformLocation(self.shader, key)
            if len(data) == 1:
                glUniform1f(loc, data[0]) 
            elif len(data) == 2:
                glUniform2f(loc, data[0], data[1])

        glBindVertexArray(self.render_rect.vao)
        glDrawArrays(GL_TRIANGLES, 0, self.render_rect.vertex_count)


from OpenGL.GL import *
from OpenGL.GL.shaders import compileProgram,compileShader

def create_shader(vertex_filepath, fragment_filepath):
    with open(vertex_filepath,'r') as f:
        vertex_src = f.readlines()

    with open(fragment_filepath,'r') as f:
        fragment_src = f.readlines()
    
    shader = compileProgram(compileShader(vertex_src, GL_VERTEX_SHADER),
                            compileShader(fragment_src, GL_FRAGMENT_SHADER))
    
    return shader


import numpy as np
from OpenGL.GL import *
import ctypes

class ScreenRect:
    def __init__(self, size, win_size, offset):
        self.size = size
        offset = (offset[0]/win_size[0], offset[1]/win_size[1])

        self.current_w, self.current_h = win_size
        
        x = self.size[0] / self.current_w
        y = self.size[1] / self.current_h

        self.vertices = (
            -x + offset[0],  y + offset[1],    0.0, 1.0, 0.0, 0.0,    0.0, 1.0,
             x + offset[0],  y + offset[1],    0.0, 1.0, 0.0, 0.0,    1.0, 1.0,
            -x + offset[0], -y + offset[1],    0.0,1.0, 0.0, 0.0,     0.0, 0.0,

           -x + offset[0], -y + offset[1],     0.0,1.0, 0.0, 0.0,      0.0, 0.0,
           x + offset[0],  y + offset[1],      0.0,1.0, 0.0, 0.0,       1.0, 1.0,
           x + offset[0], -y + offset[1],      0.0,1.0, 0.0, 0.0,       1.0, 0.0,
        )

        self.vertices = np.array(self.vertices, dtype=np.float32)

        self.vertex_count = 6

        self.vao = glGenVertexArrays(1)
        glBindVertexArray(self.vao)
        self.vbo = glGenBuffers(1)
        glBindBuffer(GL_ARRAY_BUFFER, self.vbo)
        glBufferData(GL_ARRAY_BUFFER, self.vertices.nbytes, self.vertices, GL_STATIC_DRAW)

        glEnableVertexAttribArray(0)
        glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, 32, ctypes.c_void_p(0))
        
        glEnableVertexAttribArray(1)
        glVertexAttribPointer(1, 3, GL_FLOAT, GL_FALSE, 32, ctypes.c_void_p(12))

        glEnableVertexAttribArray(2)
        glVertexAttribPointer(2, 2, GL_FLOAT, GL_FALSE, 32, ctypes.c_void_p(24))


from OpenGL.GL import *
import os
os.environ['PYGAME_HIDE_SUPPORT_PROMPT'] = "hide"
import pygame

class Texture:
    def __init__(self, surface):
        self.texture = glGenTextures(1)
        glBindTexture(GL_TEXTURE_2D, self.texture)
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT)
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT)
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST)
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST)

        image = surface
        image = pygame.transform.flip(image, False, True)
        image_width,image_height = image.get_rect().size
        img_data = pygame.image.tostring(image,'RGBA')
        glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,image_width,image_height,0,GL_RGBA,GL_UNSIGNED_BYTE,img_data)
        glGenerateMipmap(GL_TEXTURE_2D)

    def use(self):
        glActiveTexture(GL_TEXTURE0)
        glBindTexture(GL_TEXTURE_2D,self.texture)